#!/bin/bash

LOGPATH="./"
LOGCONTEXT=$(basename $0)

MYSQL_DOCKER="mysqlTEST"
MYSQL_TAG=""

MYSQL_TAGS=()
MYSQL_TAGS+=($(echo 5.5.{50..60}))
MYSQL_TAGS+=($(echo 5.6.{30..40}))
MYSQL_TAGS+=($(echo 5.7.{10..22}))
#MYSQL_TAGS=()
#MYSQL_TAGS+=('5.5.51')
#MYSQL_TAGS+=('5.7.22')

MYSQL_RESULTS=()

MYSQLCMD="docker exec -i $MYSQL_DOCKER mysql"
DB_NAME="test"

#test files
STEP1FN="step1.sql"
STEP2aFN="step2a.sql"
STEP2bFN="step2b.sql"
STEP3FN="step3ff.sql"
RUNS=30


#################################################
## do logging
## $1
## $2
function writelog
{
        logTime=$(date +"%d.%m.%Y %T")
        echo -e "[$logTime] [$LOGCONTEXT] $1 $2"
}

#################################################
## start db
function startDB
{
	writelog [INF] "Start DB"
	# docker -l warn run --name mysqlTEST -e MYSQL_ALLOW_EMPTY_PASSWORD=yes -d mysql:5.7.22 
	docker -l warn run --name $MYSQL_DOCKER -e MYSQL_ALLOW_EMPTY_PASSWORD=yes -d mysql:$MYSQL_TAG > /dev/null
	if [[ $? -ne 0 ]] ; then
		writelog [ERR] "Unable create docker mysql"
		return 1
	fi 
	
	writelog [INF] "Waiting for database connection"
	# we use the IP here as the db is startet twice and only after second it is realy up
	while ! $MYSQLCMD --table -h 127.0.0.1 -e "SHOW GLOBAL VARIABLES LIKE '%version%';" 2> /dev/null ; do
		echo -n "."
		sleep 1
	done
	echo ""
}

#################################################
## start db
function stopDB
{
	writelog [INF] "stop DB"
	docker stop $MYSQL_DOCKER > /dev/null
	
	writelog [INF] "remove docker"	
	docker rm $MYSQL_DOCKER > /dev/null
}


#################################################
## prepare db
function prepareTest
{
	writelog [INF] "Preparing DB (Step 1)"
	
	$MYSQLCMD -e "CREATE DATABASE IF NOT EXISTS $DB_NAME"
	if [[ $? -ne 0 ]] ; then
		writelog [ERROR] "Create DB faild"
		return 1
	fi
	
	$MYSQLCMD -D $DB_NAME < $STEP1FN
	if [[ $? -ne 0 ]] ; then
		writelog [ERROR] "Preparing DB faild"
		return 1
	fi
}

#################################################
## runTest db
function runTest
{
	#total errors
	etcnt=0
	
	# writelog [INF] "results before action:"
	RESULT_BEFORE=( $($MYSQLCMD -BN -D $DB_NAME < $STEP3FN) )
	writelog [INF] " state before: $(printf " %4s" "${RESULT_BEFORE[@]}")"
	
	for (( r=0; r<${RUNS}; r++ )); do
		#errors per run
		ecnt=0
		
		# writelog [INF] "run action:"
		$MYSQLCMD -D $DB_NAME < $STEP2aFN
		if [[ $? -ne 0 ]] ; then
			writelog [ERR] "Unable to run Step 2a"
			return -1
		fi
		
		$MYSQLCMD -D $DB_NAME < $STEP2bFN
		if [[ $? -ne 0 ]] ; then
			writelog [ERR] "Unable to run Step 2b"
			return -1
		fi

		# writelog [INF] "results after action:"
		RESULT_AFTER=( $($MYSQLCMD -BN -D $DB_NAME < $STEP3FN) )
		
		aLen=${#RESULT_BEFORE[@]}
		for (( i=0; i<${aLen}; i++ )); do
			if [ "${RESULT_BEFORE[$i]}" -ne "${RESULT_AFTER[$i]}" ]; then
				ecnt=$(( $ecnt + 1 ))
				etcnt=$(( $etcnt + 1 ))
			fi
		done
		
		if [[ $ecnt -ne 0 ]] ; then
			writelog [ERR] "run $(printf "%2d" "$r") result: $(printf " %4s" "${RESULT_AFTER[@]}") FAILED($ecnt)"
		else
			writelog [INF] "run $(printf "%2d" "$r") result: $(printf " %4s" "${RESULT_AFTER[@]}") OK"
		fi 
		
	done
	
	return $etcnt
}

# stop previous faild run
LOGCONTEXT="stop previous"
stopDB
writelog [INF] "***********************"

for MYSQL_TAG in "${MYSQL_TAGS[@]}" ; do
	LOGCONTEXT=$MYSQL_TAG
	
	writelog [INF] "Start Test"
	startDB
	if [[ $? -ne 0 ]] ; then
		writelog [WRN] "Unable to create DB"
		continue
	fi 
	
	TSTART=$(date +%s)
	
	prepareTest
	if [[ $? -ne 0 ]] ; then
		writelog [WRN] "Unable to prepare Test"
		continue
	fi 
	
	runTest
	if [[ $? -ne 0 ]] ; then
		RESULT="$(printf "%6s %3d %s" "FAILED" "$etcnt" "times")"
	else
		RESULT="$(printf "%6s" "OK")"
	fi
	
	TEND=$(date +%s)
	TRUN=$((TEND-TSTART))
	MYSQL_RESULTS+=("$(printf "%7s, %3d, %s" "$MYSQL_TAG" "$TRUN" "$RESULT")")
	
	stopDB
	
	writelog [INF] "***********************"
done

LOGCONTEXT="TOTAL"
writelog [INF] "Total result:"
echo "Version, sec, Result"
aLen=${#MYSQL_RESULTS[@]}
for (( i=0; i<${aLen}; i++ )); do
	echo "${MYSQL_RESULTS[$i]}"
done




